"""
These template tags are for converting from srid 2100 to the one that google is
using.
"""

from django.template import Library, Node, TemplateSyntaxError
from pyproj import Proj, transform
from enhydris.conf import settings
from enhydris.hcore.models import Station

register = Library()

def get_longitude(station_id):
    """
    Get station longitude in 4326 srid
    """

    try:
        station = Station.objects.get(id=station_id)
    except:
        return None

    if station.abscissa == station.ordinate == None:
        return None

    p1 = Proj(init='epsg:2100')
    p2 = Proj(init='epsg:4326')
    (latitude, longitude) = transform(p1, p2, station.abscissa,
                                        station.ordinate)

    return longitude


def get_latitude(station_id):
    """
    Get station latitude in 4326 srid
    """

    try:
        station = Station.objects.get(id=station_id)
    except:
        return None

    p1 = Proj(init='epsg:2100')
    p2 = Proj(init='epsg:4326')

    if station.abscissa == station.ordinate == None:
        return None

    (latitude, longitude) = transform(p1, p2, station.abscissa,
                                        station.ordinate)

    return latitude


register.simple_tag(get_longitude)
register.simple_tag(get_latitude)



"""
Management command which takes care of syncing the master db
"""

import datetime
import inspect
import os
import re
import signal
import sys
import threading
import urllib2
import json
from optparse import make_option
from socket import socket, gethostbyname, AF_INET, SOCK_STREAM

from django.core import serializers
from django.core.management.base import BaseCommand
from django.db import connection, transaction
from django.db import models as dj_models
from django.db.models import get_app
from django.utils.http import urlquote

from enhydris.dbsync.models import Database


# Register a signal to catch kbd interrupts and handle them
def signal_handler(signal, frame):
    """
    Signal handler for keyboard interrupts (ctrl+C)
    """
    MSG("Keyboard Interrupt received. Aborting...")
    gracefull_exit(1)
signal.signal(signal.SIGINT, signal_handler)


def gracefull_exit(status):
    """
    This handles system cleanup in case of an exception or user given
    interrupt.
    """
    # Ignore any more signals (HUP mostly)
    signal.signal(signal.SIGINT, signal.SIG_IGN)
    # Get object list
    onames = [ f for f in globals().keys() if re.match("^Generic.*", f) ]
    onames.sort(reverse=True)

    transaction.commit_unless_managed()
    transaction.enter_transaction_management()
    transaction.managed(True)
    # delete the objects
    for object in onames:
        try:
            exec(object+'.delete()')
        except Exception, detail:
            pass

    transaction.commit()
    transaction.leave_transaction_management()

    # exit
    sys.exit(status)


# This dict holds all outstanding foreign key relations that could not be added
# during the first transaction. This wil be parsed after the main transaction
# is commited and all objects are saved in the local db

batch_jobs = {}


# A progress indicator just for presentation
class RotatingThing(threading.Thread):
    """
    This is a progress indicator which displays a rotating thing.
    """
    def __init__(self, msg):
        threading.Thread.__init__(self)
        self.msg = msg
        self.array = '-\|/'
        self.current = 0
        self.event = threading.Event()
    def __enter__(self):
        self.start()
    def __exit__(self, ex_type, ex_value, ex_traceback):
        self.event.set()
        sys.stdout.write("\b: done\n")
        self.join()
    def run(self):
        """
        This is the function that gets executed when the thread starts
        """
        sys.stdout.write(self.msg)
        while not self.event.isSet():
            sys.stdout.write("\b%s" % self.array[self.current % \
                                    len(self.array)])
            self.current += 1
            sys.stdout.flush()
            self.event.wait(0.5)


def check_remote(remote, port):
    """
    Given a hostname/ip and a port number, the function checks to see if there
    is any application listening at the other side of the connection.
    """

    # Get host ip
    if not re.match('(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25'\
                    '[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)', remote):
        try:
            remoteIP = gethostbyname(remote)
        except:
            ERRMSG('Remote address doesn\'t seem valid. Make sure you used'\
                   ' a valid address and that the remote server is running')
            gracefull_exit(1)
    else:
        ERRMSG('Please provide a FQDN for the remote instance and not just'\
               ' the ip. Exiting...')
        gracefull_exit(1)
    # Get port
    try:
        port = int(port)
        if port <= 0 or port > 65536:
            raise Exception
    except:
        ERRMSG('The port you have specified doesn\'t seem to be valid.')
        return False

    # Check if host is accessible
    MSG('Checking port availability on host %s, port %s' % (remoteIP, port))
    s = socket(AF_INET, SOCK_STREAM)
    try:
        s.connect((remoteIP, port))
    except:
        s.close()
        return False

    s.close()
    return True


def get_models(app, exc_list=None):
    """
    This function returns a list of models given a specific application.

    To do this, we take the app name and inspect the app.models module and we
    extract all necessary info from it.
    """

    # empty model list
    model_list = []
    # get the models of the app
    app_models = dj_models.get_models(app)
    # remove excluded models
    for item in app_models:
        name = item.__name__
        try:
            exc_list.index(name)
        except ValueError:
            model_list.append(item)

    return model_list


def sort_by_dep(model_list):
    """
    This function sorts a list of models and their dependencies to calculate
    the order in which the models should be loaded by the deserializer
    """

    sorted_list = [ model.__name__ for model in  model_list ]
    deps = []
    for model in model_list:
        if hasattr(model, 'f_dependencies'):
            deps = getattr(model, 'f_dependencies')
            for dep in deps:
                if dep:
                    try:
                        if sorted_list.index(dep) > \
                           sorted_list.index(model.__name__):
                            sorted_list.remove(dep)
                            sorted_list.insert(sorted_list.index(model.__name__), dep)
                    except ValueError:
                        ERRMSG("A model which is a dependency of other"\
                               " objects is not included in the sync. This may"\
                               " cause the sync to fail. Exiting...")
                        gracefull_exit(1)

    return sorted_list


def create_generic_objects(app):
    """
    This function creates a set of generic objects which hold temporary Not Null
    foreign keys for objects which are being installed.
    """

    transaction.commit_unless_managed()
    transaction.enter_transaction_management()
    transaction.managed(True)
#
#    for model in models:
#        mfields = model._meta.fields
#        nn = [ f.name for f in mfields if not f.null and not f.blank ]
#        for field in nn:
#            (type, a, b, c) = model._meta.get_field_by_name(field)
#            if type.__class__.__name__ == 'ForeignKey'\
#                or type.__class__.__name__ == 'OneToOneField':
#                fkey = type.related.parent_model.__name__
#                exec("f_val = Generic"+fkey)
#            else:
#                f_val = 0
#        exec("Generic"+model.__name__+"=models."+model.__name__+"()")
#

    globals()["GenericGarea"] = app.Garea()
    GenericGarea.save()
    globals()["GenericGentity"] = app.Gentity()
    GenericGentity.save()
    globals()["GenericGentityAltCodeType"] = app.GentityAltCodeType()
    GenericGentityAltCodeType.save()
    globals()["GenericGentityAltCode"] = app.GentityAltCode(gentity=GenericGentity, type
            = GenericGentityAltCodeType )
    GenericGentityAltCode.save()
    globals()["GenericFileType"] = app.FileType()
    GenericFileType.save()
    globals()["GenericEventType"] = app.EventType()
    GenericEventType.save()
    globals()["GenericGentityEvent"] = app.GentityEvent(gentity=GenericGentity, type=
                GenericEventType, date = datetime.datetime.now())
    GenericGentityEvent.save()
    #GenericGentityFile = app.GentityFile()

    globals()["GenericGline"] = app.Gline()
    GenericGline.save()

    globals()["GenericGpoint"] = app.Gpoint()
    GenericGpoint.save()
    globals()["GenericInstrumentType"] = app.InstrumentType()
    GenericInstrumentType.save()
    globals()["GenericLentity"] = app.Lentity()
    GenericLentity.save()
    globals()["GenericStation"] = app.Station(owner=GenericLentity)
    GenericStation.save()
    globals()["GenericInstrument"] = app.Instrument(station=GenericStation, type=
                GenericInstrumentType)
    GenericInstrument.save()
    globals()["GenericPerson"] = app.Person()
    GenericPerson.save()
    globals()["GenericVariable"] = app.Variable()
    GenericVariable.save()

    globals()["GenericUnitOfMeasurement"] = app.UnitOfMeasurement()
    GenericUnitOfMeasurement.save()
    globals()["GenericTimeZone"] = app.TimeZone(utc_offset="+2")
    GenericTimeZone.save()

    globals()["GenericTimeStep"] = app.TimeStep(length_minutes=0,
                            length_months=1)
    GenericTimeStep.save()
    globals()["GenericWaterBasin"] = app.WaterBasin()
    GenericWaterBasin.save()
    globals()["GenericWaterDivision"] = app.WaterDivision()
    GenericWaterDivision.save()
    globals()["GenericPoliticalDivision"] = app.PoliticalDivision()
    GenericPoliticalDivision.save()

    transaction.commit()
    transaction.leave_transaction_management()


def store_fkeys(app_name, model_list, object, m2m_keys='None'):
    """
    This function iterates through the fields of the deserialized object and
    finds all foreign keys. After that, by using the original id, he makes sure
    that the foreign keys point to the correct objects.
    """

    foreign_keys = [ f.name for f in object._meta.fields if
                                    isinstance(f, dj_models.ForeignKey) and not
                                    f.name == 'original_db' ]

    if hasattr(object.__class__, 'f_dependencies'):
        deps = getattr(object.__class__, 'f_dependencies')
        parent = deps[0]
    else:
        parent = None

    # get model of parent


    if foreign_keys or m2m_keys:
        try:
            batch_jobs[object.__class__].update( { object.original_id: {
                                        'oid':object.original_id,
                                        'f_keys': {},
                                        'm2m': m2m_keys } })
        except KeyError:
            batch_jobs[object.__class__] = { object.original_id: {
                                        'oid':object.original_id,
                                        'f_keys': {},
                                        'm2m': m2m_keys } }

	# iterate all keys and save them & then make them None
    # keep only inheritance relationship intact
    for key in foreign_keys:
        if parent and key == parent.lower()+'_ptr':
            model = dj_models.get_model(app_name, parent)
            parent_id = getattr(object, key+'_id')
            # get parent object
            po = model.objects.get(original_id=parent_id, original_db=DB)
            # assing new parent pointer
            setattr(object, key, po)
        else:
            key_val = getattr(object, key+'_id')
            if key_val:
                try:
                    fobj = getattr(object, key)
                    go_name = fobj.__class__.__name__
                except:
                    #remote object doesn't exist yet
                    ff = [ f for (f, m) in object._meta.get_fields_with_model()
                                         if f.name == key][0]
                    go_name = ff.related.parent_model.__name__
                # only keep the foreign key if remote object is in the sync
                # list. Otherwise wipe this out.
                if go_name in model_list:
                    batch_jobs[object.__class__][object.original_id]['f_keys'].update({key: key_val})

                try:
                    setattr(object, key, None)
                except ValueError:
                    #Some attributes don't allow Null. Now what?
                    setattr(object, key, eval('Generic'+go_name))


def eval_fkeys(app_name, models):
    """
    This function iterates through all the entries in the ``batch_jobs''
    dictionary and re-evaluates all foreign keys for new objects using the
    original id of the object.
    """

    for model in batch_jobs.keys():
        model_name = model.__name__

        for object_id in batch_jobs[model].keys():
            item = model.objects.get(original_id=str(object_id), original_db=DB)

            # foreign
            if batch_jobs[model][object_id].has_key('f_keys'):
                for (key, val) in batch_jobs[model][object_id]['f_keys'].iteritems():
                    ff = [ f for (f, m) in model._meta.get_fields_with_model() if
                                        f.name==key][0]
                    ro_name = ff.related.parent_model.__name__
                    f_model = dj_models.get_model(app_name, ro_name)
                    setattr(item, key, f_model.objects.get(original_id=val,
                                                            original_db=DB))
            # m2m
            if batch_jobs[model][object_id].has_key('m2m'):
                for (key, vals) in batch_jobs[model][object_id]['m2m'].iteritems():
                    remote_obj = getattr(item, key)
                    remote_class = remote_obj.model.__name__
                    for val in vals:
                        f_model = dj_models.get_model(app_name, remote_class)
                        ro = f_model.objects.get(original_id=val, original_db=DB)
                        m2m_handler = getattr(item, key)
                        if hasattr(m2m_handler, 'add'):
                            m2m_handler.add(ro)
                            # If it doesn't have 'add' it means it is a m2m
                            # with a "through". We don't fix that; instead, we
                            # pray it works.

            item.save()


def MSG(msg, verbose=1):
    """
    STDOUT logging function
    """
    if verbose:
        sys.stdout.write('%s\n' % msg)


def ERRMSG(msg, verbosity=1):
    """
    STDERR logging function
    """
    sys.stderr.write('--> ERROR: %s\n' % msg)


class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('--remote', '-r', dest='remote',
            help='Remote instance to sync from'),
        make_option('--port', '-p', dest='port',default=80,
            help='Specify custom port. Default is 80.'),
        make_option('--app', '-a', dest='app',
            help='Application which should be synced'),
        make_option('--exclude', '-e', dest='exclude', default=None,
            help='State which models of the apps you want excluded from the'\
                 ' sync'),
        make_option('--fetch-only', '-f', dest='fetch',default=False,
            action='store_true',
            help='Doesn\'t actually submit any changes, just fetches remote'\
                 ' dumps and saves them locally.'),
        make_option('--work-dir', '-w', dest='cwd', default='/tmp',
            help='Define the temp dir in which all temp files will be stored'),
        make_option('--full', '-F', dest='full', default=False,
            action='store_true',
            help='Bypasses last sync date and syncs all of the db again'),
                make_option('--resume', '-R', dest='resume', default=False,
            action='store_true',
            help='With resume, no files are fetched but the local ones are used'),
        make_option('--silent', '-S', dest='verbose', default=True,
            action='store_false',
            help='Suppress all log messages',)
    )
    help = 'This command is used to synchronize the local database using data'\
           ' from a remote instance'

    def handle(self, *args, **options):

        # Get a cursor and init the db
        cursor = connection.cursor()

        # keep record of installed stuff
        inst_models = set()
        fixture_count = 0
        object_count = 0

        # Save command line opts
        remote = options.get('remote', None)
        port = options.get('port')
        app_name = options.get('app', None)
        exclude = options.get('exclude', None)
        pretend = options.get('pretend')
        ask = options.get('ask')
        interactive = options.get('interactive')
        clean = options.get('clean')
        full = options.get('full')
        no_backup = options.get('bkp')
        verbose = options.get('verbose')
        fetch = options.get('fetch')
        resume = options.get('resume')
        cwd = options.get('cwd')

        # check if we have a remote node and an app
        if not remote or not app_name:
            ERRMSG('You must specify at least one remote system and app to'\
                   ' sync')
            sys.exit(1)

        # handle silent option
        if not verbose:
            null = file("/dev/null", "a")
            sys.stdout = sys.stderr = null

        # Check if remote node is listening
        if check_remote(remote, port):
            MSG('Remote host is up. Continuing with the sync.')
        else:
            ERRMSG('Connection error. Make sure the remote application is'\
                   ' running and that you specified the correct connection'\
                   ' parameters.')
            sys.exit(1)

        # check if remote instance is in Databases
        try:
            globals()["DB"] = Database.objects.get(hostname__contains=remote)
        except Exception:
            ERRMSG('The remote host you have provided is not in the database'\
                   ' table of our application. Please insert the host via the'\
                   ' admin panel and then try again.')
            gracefull_exit(1)

        # import the app's models
        try:
            app = get_app(app_name)
        except ImproperlyConfigured:
            ERRMSG('Could not load application %s. Exiting...' % app)
            sys.exit(1)

        if exclude:
            exc_list = exclude.split(',')
        else:
            exc_list = []


        # try to get a model list
        try:
            models = get_models(app, exc_list)
        except:
            ERRMSG('Could not get a list of models inside the specified app.'\
                   ' This should not happen...')
            sys.exit(1)

        # exclude specified models
        MSG("The following models will be synced: %s" %
                [m.__name__ for m in models] )
        if exclude:
            MSG("The following models will be excluded %s" %
                [m for m in exc_list] )

        # change cwd
        try:
            os.chdir(cwd)
        except OSError:
            ERR_MSG("Could not chdir to %s. You could provide another custon"\
                    " temp directory using the -w switch." % cwd)

        # get the sync timestamp
        sync_timestamp = datetime.datetime.now()

        if not resume:
            # Get the fixture for each model
            for model in models:
                MSG("Syncing model %s" % model.__name__)
                request_url = ('http://%s:%s/api/%s/' %
                                 (remote, str(port),model.__name__))
                try:
                    if DB.last_sync and not full:
                        timestamp = urlquote(str(DB.last_sync))
                        req = urllib2.Request("%smodified_after/%s/" % (request_url,
                                                                        timestamp))
                    else:
                        req = urllib2.Request(request_url)

                    with RotatingThing(" - Downloading %s fixtures  " % \
                                                model.__name__):
                        response = urllib2.urlopen(req)
                except Exception, detail:
                    ERRMSG("Error fetching fixtures. %s" % detail)

                j = self.backward_api(response, app_name, model)

                # and save it to a local file
                try:
                    local_file = open(model.__name__+'.json', "w")
                    json.dump(j, local_file)
                    local_file.close()
                except:
                    ERRMSG('Error saving local file %s. Check cwd permissions and'\
                           ' try again.' %model.__name__+'.json')

            # If fetch-only run was specified, exit now
            if fetch:
                MSG("\nFinished fetching files. Exiting now...")
                gracefull_exit(0)

        # Create Generic objects to handle temporary dependencies
        MSG("Creating Generic objects")
        create_generic_objects(app)
        MSG("Finished with Generic objects")

        # transaction management
        transaction.commit_unless_managed()
        transaction.enter_transaction_management()
        transaction.managed(True)

        # sort models by dependencies so that deps will be loaded first
        models = sort_by_dep(models)

        # Open the files on by one and load the fixtures in the db
        for model in models:
            try:
                fd = file(model+'.json', 'r')
            except IOError:
                ERRMSG("Error opening file %s." % (model+'.json'))
                gracefull_exit(1)

            MSG("Installing fixtures from file %s.json" % model)

            # do the object deserialization
            try:
                objects = serializers.deserialize('json', fd)
            except:
                ERRMSG("Error deserializing objects for class %s." % \
                        model)
                objects = None
                gracefull_exit(1)

            # handle each deserialized object
            for obj in objects:
                inst_models.add(obj.object.__class__)

                try:
                    obj_type = dj_models.get_model(app_name, obj.object.__class__.__name__)
                    upd_obj = obj_type.objects.get(original_id=obj.object.pk, original_db=DB)
                except Exception, detail:
                    # not already in db
                    obj.object.original_id = obj.object.pk
                    obj.object.id = None
                else:
                    # object in db. updating...
                    obj.object.id = upd_obj.id
                    obj.object.original_id = upd_obj.original_id

                # Here we must also populate the additional fields and
                # check every addition against the existing objects!
                store_fkeys(app_name, models, obj.object, obj.m2m_data)

                obj.m2m_data = {}
                obj.object.original_db = DB

                try :
                    obj.save()
                    object_count += 1
                except Exception, details:
                    ERRMSG("Error saving an object of type %s\n%s" %\
                            (model, details) )
                    transaction.rollback()
                    transaction.leave_transaction_management()
                    gracefull_exit(1)


            fd.close()
            os.unlink(fd.name)
            fixture_count += 1

        try:
            with RotatingThing("Reinitializing foreign keys:  "):
                eval_fkeys(app_name, models)
        except Exception, details:
            ERRMSG("Error setting up foreign keys: %s" % details)
            transaction.rollback()
            transaction.leave_transaction_management()
            gracefull_exit(1)

        # transaction end
        transaction.commit()
        transaction.leave_transaction_management()

        MSG("Successfully installed %d objects from %d fixtures." %
                (object_count, fixture_count) )

        # close connection
        connection.close()

        # update DB last sync timestamp
        setattr(DB, 'last_sync', sync_timestamp)
        DB.save()

        # wrap-up and exit
        gracefull_exit(0)

    def backward_api(self, response, app_name, model):
        """
        The dbsync application was originally written for some kind of
        webservice API we had then. Either that API was compatible
        with the Django deserializer, or I don't understand something
        (or both).  The API since has changed. So here we hackily
        convert the results given by the new API so that it's like the
        results of the old API.  In the old API, each item had a "pk",
        a "model", and a "fields" dictionary with all the other
        fields. In the new API, there is an "id", there is no "model"
        (we know which model it is), and all the other fields are
        there, siblings of "id" (not nested inside a "fields"
        dictionary).

        If results are already in the old API, we don't touch it.
        """
        result = []
        try:
            j = json.load(response)
        except ValueError:
            return result
        if len(j) < 1:
            return result
        if ('pk' in j[0]) and ('model' in j[0]) and ('fields' in j[0]) and (
                'id' not in j[0]):
            # Old API: return as is
            result = j
            return result
        for x in j:
            item = {
                'pk': x['id'],
                'model': app_name + '.' + model.__name__,
                'fields': {},
            }
            for k in x:
                if x == 'id':
                    continue
                item['fields'][k] = x[k]
            result.append(item)
        return result



from django import template
register = template.Library()

def dict_get(value, arg):
    #custom template tag used like so:
    #{{dictionary|dict_get:var}}
    #where dictionary is duh a dictionary and var is a variable representing
    #one of it's keys
    if value:
        return value.get(arg, None)

    return None

def list_empty(list):
    """
    Check if list is empty
    """

    for item in list:
        if item:
            return False
    return True


register.filter('dict_get',dict_get)
register.filter('list_empty',list_empty)



from datetime import datetime
import json

from django.contrib.auth.models import User, Permission
from django.db import connection
from django.test import TestCase
from django.test.client import MULTIPART_CONTENT, BOUNDARY, encode_multipart
from django.test.utils import override_settings

from rest_framework.test import APITestCase

from pthelma.timeseries import Timeseries
from enhydris.hcore import models


def get_gentities_from_fixture():
    """
    This function is used internally by ReadTestCase.testGentityList.
    It gets the fixture data, gets only the gentities from there,
    flattens it, and converts the datetime field to a datetime. The result
    should be a list of dictionaries identical to the one the API returns when
    it is queried for the list of gentities.
    """
    fixture_data = json.load(open('enhydris/api/testdata.json'))
    result = []
    for item in fixture_data:
        if item['model'] != 'hcore.gentity':
            continue
        nitem = {}
        nitem['id'] = item['pk']
        for key in item['fields']:
            if key == 'model':
                continue
            nitem[key] = item['fields'][key]
            if key == 'last_modified':
                nitem[key] = datetime.strptime(nitem[key],
                                               '%Y-%m-%d %H:%M:%S')
        result.append(nitem)
    result.sort(key=lambda x: x['id'])
    return result


class ReadTestCase(APITestCase):
    fixtures = ['enhydris/api/testdata.json']

    def testGentityList(self):

        response = self.client.get("/api/Gentity/")
        response.data.sort(key=lambda x: x['id'])
        reference_data = get_gentities_from_fixture()
        self.assertEqual(response.data, reference_data)

        # Same thing, but test with modified_after
        n_all_gentities = len(reference_data)
        for item in reference_data[:]:
            if item['last_modified'] < datetime(2010, 5, 11):
                reference_data.remove(item)
        n_recent_gentities = len(reference_data)
        self.assertTrue(n_recent_gentities < n_all_gentities)
        response = self.client.get(
            '/api/Gentity/modified_after/2010-05-11 00:00/')
        response.data.sort(key=lambda x: x['id'])
        self.assertEqual(response.data, reference_data)


class WriteTestCase(TestCase):
    fixtures = ['enhydris/api/testdata.json']

    def testTimeSeries(self):
        # Get an existing time series
        obj = models.Timeseries.objects.filter(name='Test Timeseries')[0]
        response = self.client.get("/api/Timeseries/%d/" % (obj.id,))
        t = json.loads(response.content)

        # Change some of its attributes
        t['id'] = None
        t['name'] = "Test Timeseries 1221"
        t['remarks'] = "Yet another timeseries test"

        # Attempt to upload unauthenticated - should deny
        d = json.dumps(t)
        response = self.client.post("/api/Timeseries/", data=d,
                                    content_type='application/json')
        self.assertEqual(response.status_code, 403)
        self.assertEqual(models.Timeseries.objects.filter(
                         name='Test Timeseries 1221').count(), 0)

        # Now try again, this time logged on as user 2; again should deny
        self.assert_(self.client.login(username='user2', password='password2'))
        response = self.client.post("/api/Timeseries/", data=d,
                                    content_type='application/json')
        self.assertEqual(response.status_code, 403)
        self.assertEqual(models.Timeseries.objects.filter(
                         name='Test Timeseries 1221').count(), 0)
        self.client.logout()

        # Now try again, this time logged on as user 1; should accept
        self.assert_(self.client.login(username='user1', password='password1'))
        response = self.client.post("/api/Timeseries/", data=d,
                                    content_type='application/json')
        self.assertEqual(response.status_code, 201)
        self.assertEqual(models.Timeseries.objects.filter(
                         name='Test Timeseries 1221').count(), 1)
        self.client.logout()

    def testTimeseriesDelete(self):
        # Check the number of timeseries available
        ntimeseries = models.Timeseries.objects.count()
        assert(ntimeseries > 1)  # 1 is not enough; we need to know we aren't
                                 # deleting more than necessary.

        # Attempt to delete unauthenticated - should fail
        response = self.client.delete("/api/Timeseries/1/")
        self.assertEqual(response.status_code, 403)
        self.assertEqual(models.Timeseries.objects.filter(pk=1).count(), 1)
        self.assertEqual(models.Timeseries.objects.count(), ntimeseries)

        # Try again as user2 - should fail
        self.assert_(self.client.login(username='user2', password='password2'))
        response = self.client.delete("/api/Timeseries/1/")
        self.assertEqual(response.status_code, 403)
        self.assertEqual(models.Timeseries.objects.filter(pk=1).count(), 1)
        self.assertEqual(models.Timeseries.objects.count(), ntimeseries)
        self.client.logout()

        # Try again as user1 - should succeed
        self.assert_(self.client.login(username='user1', password='password1'))
        response = self.client.delete("/api/Timeseries/1/")
        self.assertEqual(response.status_code, 204)
        self.assertEqual(models.Timeseries.objects.filter(pk=1).count(), 0)
        self.assertEqual(models.Timeseries.objects.count(), ntimeseries - 1)
        self.client.logout()

    def testUploadTsDataUnauthenticated(self):
        # Attempt to upload some timeseries data, unauthenticated
        response = self.client.put(
            "/api/tsdata/1/",
            encode_multipart(BOUNDARY,
                             {'timeseries_records': '2012-11-06 18:17,20,\n'}),
            content_type=MULTIPART_CONTENT)
        t = Timeseries(1)
        t.read_from_db(connection)
        self.assertEqual(response.status_code, 403)
        self.assertEqual(len(t), 0)

    def testUploadTsDataAsWrongUser(self):
        # Attempt to upload some timeseries data as user 2; should deny
        self.assert_(self.client.login(username='user2', password='password2'))
        response = self.client.put(
            "/api/tsdata/1/",
            encode_multipart(BOUNDARY,
                             {'timeseries_records': '2012-11-06 18:17,20,\n'}),
            content_type=MULTIPART_CONTENT)
        t = Timeseries(1)
        t.read_from_db(connection)
        self.assertEqual(response.status_code, 403)
        self.assertEqual(len(t), 0)
        self.client.logout()

    def testUploadTsDataGarbage(self):
        self.assert_(self.client.login(username='user1', password='password1'))
        response = self.client.put(
            "/api/tsdata/1/",
            encode_multipart(BOUNDARY,
                             {'timeseries_records': '2012-aa-06 18:17,20,\n'}),
            content_type=MULTIPART_CONTENT)
        t = Timeseries(1)
        t.read_from_db(connection)
        self.assertEqual(response.status_code, 400)
        self.assertEqual(len(t), 0)
        self.client.logout()

    def testUploadTsData(self):
        self.assert_(self.client.login(username='user1', password='password1'))
        response = self.client.put(
            "/api/tsdata/1/",
            encode_multipart(BOUNDARY,
                             {'timeseries_records': '2012-11-06 18:17,20,\n'}),
            content_type=MULTIPART_CONTENT)
        t = Timeseries(1)
        t.read_from_db(connection)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, '1')
        self.assertEqual(len(t), 1)
        self.assertEqual(t.items(0)[0], datetime(2012, 11, 06, 18, 17, 0))
        self.assertEqual(t.items(0)[1], 20)
        self.client.logout()

        # Append two more records
        self.assert_(self.client.login(username='user1', password='password1'))
        response = self.client.put(
            "/api/tsdata/1/",
            encode_multipart(BOUNDARY,
                             {'timeseries_records':
                              '2012-11-06 18:18,21,\n2012-11-07 18:18,23,\n'}),
            content_type=MULTIPART_CONTENT)
        t = Timeseries(1)
        t.read_from_db(connection)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, '2')
        self.assertEqual(len(t), 3)
        self.assertEqual(t.items(0)[0], datetime(2012, 11, 06, 18, 17, 0))
        self.assertEqual(t.items(0)[1], 20)
        self.assertEqual(t.items(1)[0], datetime(2012, 11, 06, 18, 18, 0))
        self.assertEqual(t.items(1)[1], 21)
        self.assertEqual(t.items(2)[0], datetime(2012, 11, 07, 18, 18, 0))
        self.assertEqual(t.items(2)[1], 23)
        self.client.logout()

        # Try to append an earlier record; should fail
        self.assert_(self.client.login(username='user1', password='password1'))
        response = self.client.put(
            "/api/tsdata/1/",
            encode_multipart(BOUNDARY,
                             {'timeseries_records': '2012-11-05 18:18,21,\n'}),
            content_type=MULTIPART_CONTENT)
        self.client.logout()
        t = Timeseries(1)
        t.read_from_db(connection)
        self.assertEqual(response.status_code, 409)
        self.assertEqual(len(t), 3)
        self.client.logout()


class WriteStationTestCase(TestCase):
    fixtures = ['enhydris/api/testdata.json']

    def test_edit_station(self):
        # Get an existing station
        obj = models.Station.objects.get(name='Test Station')
        response = self.client.get("/api/Station/{}/".format(obj.id))
        station = json.loads(response.content)

        # Change some of its attributes
        station_id = station['id']
        del station['id']
        station['name'] = 'Test Station 1222'
        station['remarks'] = 'Yet another station test'

        # Attempt to upload unauthenticated - should deny
        d = json.dumps(station)
        response = self.client.put('/api/Station/{}/'.format(station_id),
                                   data=d, content_type='application/json')
        self.assertEqual(response.status_code, 403)
        self.assertEqual(models.Station.objects.filter(
                         name='Test Station 1222').count(), 0)

        # Try again, this time logged on as user 2; again should deny
        self.assertTrue(self.client.login(username='user2',
                                          password='password2'))
        response = self.client.put('/api/Station/{}/'.format(station_id),
                                   data=d, content_type='application/json')
        self.assertEqual(response.status_code, 403)
        self.assertEqual(models.Station.objects.filter(
                         name='Test Station 1222').count(), 0)
        self.client.logout()

        # Try again, as user 1; should accept
        self.assertTrue(self.client.login(username='user1',
                                          password='password1'))
        response = self.client.put('/api/Station/{}/'.format(station_id),
                                   data=d, content_type='application/json')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(models.Station.objects.filter(
                         name='Test Station 1222').count(), 1)
        self.client.logout()

    def test_station_delete(self):
        # Check the number of stations available
        nstations = models.Station.objects.count()
        assert(nstations > 1)  # 1 is not enough; we need to know we aren't
                               # deleting more than necessary.

        # Attempt to delete unauthenticated - should fail
        response = self.client.delete('/api/Station/4/')
        self.assertEqual(response.status_code, 403)
        self.assertEqual(models.Station.objects.filter(pk=4).count(), 1)
        self.assertEqual(models.Station.objects.count(), nstations)

        # Try again as user2 - should fail
        self.assertTrue(self.client.login(username='user2',
                                          password='password2'))
        response = self.client.delete('/api/Station/4/')
        self.assertEqual(response.status_code, 403)
        self.assertEqual(models.Station.objects.filter(pk=4).count(), 1)
        self.assertEqual(models.Station.objects.count(), nstations)
        self.client.logout()

        # Try again as user1 - should succeed
        self.assertTrue(self.client.login(username='user1',
                                          password='password1'))
        response = self.client.delete('/api/Station/4/')
        self.assertEqual(response.status_code, 204)
        self.assertEqual(models.Station.objects.filter(pk=4).count(), 0)
        self.assertEqual(models.Station.objects.count(), nstations - 1)
        self.client.logout()


class CreateStationTestCase(TestCase):
    fixtures = ['enhydris/api/testdata.json']

    def setUp(self):
        # Get an existing station
        obj = models.Station.objects.get(name='Test Station')
        response = self.client.get("/api/Station/{}/".format(obj.id))
        self.station = json.loads(response.content)

        # Change some of its attributes
        del self.station['id']
        self.station['name'] = 'Test Station 1507'
        self.station['remarks'] = 'Yet another station test'

    def test_create_unauthenticated(self):
        response = self.client.post('/api/Station/',
                                    data=json.dumps(self.station),
                                    content_type='application/json')
        self.assertEqual(response.status_code, 403)
        self.assertEqual(models.Station.objects.filter(
                         name='Test Station 1507').count(), 0)

    @override_settings(ENHYDRIS_USERS_CAN_ADD_CONTENT=True)
    def test_create_station_on_database_allowing_any_user(self):
        self.assertTrue(self.client.login(username='user2',
                                          password='password2'))
        response = self.client.post('/api/Station/',
                                    data=json.dumps(self.station),
                                    content_type='application/json')
        self.assertEqual(response.status_code, 201)
        self.assertEqual(models.Station.objects.filter(
            name='Test Station 1507').count(), 1)
        self.client.logout()

    @override_settings(ENHYDRIS_USERS_CAN_ADD_CONTENT=False)
    def test_create_station_by_disallowed_user(self):
        self.assertTrue(self.client.login(username='user2',
                                          password='password2'))
        response = self.client.post('/api/Station/',
                                    data=json.dumps(self.station),
                                    content_type='application/json')
        self.assertEqual(response.status_code, 403)
        self.assertEqual(models.Station.objects.filter(
            name='Test Station 1507').count(), 0)
        self.client.logout()

    @override_settings(ENHYDRIS_USERS_CAN_ADD_CONTENT=False)
    def test_create_station_by_allowed_user(self):
        # Give the appropriate permissions to the user
        user = User.objects.get(username='user2')
        permission = Permission.objects.get(codename='add_station')
        user.user_permissions.add(permission)
        user.save()

        self.assertTrue(self.client.login(username='user2',
                                          password='password2'))
        response = self.client.post('/api/Station/',
                                    data=json.dumps(self.station),
                                    content_type='application/json')
        self.assertEqual(response.status_code, 201)
        self.assertEqual(models.Station.objects.filter(
            name='Test Station 1507').count(), 1)
        self.client.logout()



from django.conf.urls import patterns, url
from django.views.generic.base import RedirectView

from enhydris.hcore import views

urlpatterns = patterns(
    '',

    url(r'^$', views.StationListView.as_view(),
        name='station_list'),
    url(r'^stations/l/$', RedirectView.as_view(url='../..', permanent=True)),
    url(r'^stations/d/(?P<pk>\d+)/$', views.StationDetailView.as_view(),
        name='station_detail'),
    url(r'^stations/b/(?P<pk>\d+)/$', views.StationBriefView.as_view(),
        name='station_brief'),
    (r'^stations/edit/(?P<station_id>\d+)/$',
     views.station_edit, {}, 'station_edit'),
    (r'^stations/add/$',
     views.station_add, {}, 'station_add'),
    (r'^stations/delete/(?P<station_id>\d+)/$',
     views.station_delete, {}, 'station_delete'),

    (r'^map/$',
        views.map_view, {}, 'map_view'),

    (r'^embedmap/$',
        views.embedmap_view, {}, 'embedmap_view'),

    (r'^get_subdivision/(?P<division_id>\d+)/$',
     views.get_subdivision, {}, 'get_subdivision'),

    url(r'^instruments/d/(?P<pk>\d+)/$',
        views.InstrumentDetailView.as_view(), name='instrument_detail'),
    (r'^instrument/edit/(?P<instrument_id>\d+)/$',
     views.instrument_edit, {}, 'instrument_edit'),
    (r'^instrument/add/$', views.instrument_add, {}, 'instrument_add'),
    (r'^instrument/delete/(?P<instrument_id>\d+)/$',
     views.instrument_delete, {}, 'instrument_delete'),

    url(r'^timeseries/d/(?P<pk>\d+)/$', views.TimeseriesDetailView.as_view(),
        name='timeseries_detail'),
    (r'^timeseries/data/$',
     views.timeseries_data, {}, 'timeseries_data'),
    (r'^timeseries/d/(?P<object_id>\d+)/download/'
     r'((?P<start_date>[^/]*)/((?P<end_date>[^/]*)/)?)?$',
     views.download_timeseries, {}, 'timeseries_text'),
    (r'^timeseries/d/(?P<object_id>\d+)/bottom/$',
     views.timeseries_bottom, {}, 'timeseries_bottom'),
    (r'^timeseries/edit/(?P<timeseries_id>\d+)/$',
     views.timeseries_edit, {}, 'timeseries_edit'),
    (r'^timeseries/add/$', views.timeseries_add, {}, 'timeseries_add'),
    (r'^timeseries/delete/(?P<timeseries_id>\d+)/$',
     views.timeseries_delete, {}, 'timeseries_delete'),

    (r'^gentityfile/(?P<gf_id>\d+)/download/$',
     views.download_gentityfile, {}, 'gentityfile_dl'),
    (r'^gentityfile/edit/(?P<gentityfile_id>\d+)/$',
     views.gentityfile_edit, {}, 'gentityfile_edit'),
    (r'^gentityfile/add/$', views.gentityfile_add, {}, 'gentityfile_add'),
    (r'^gentityfile/delete/(?P<gentityfile_id>\d+)/$',
     views.gentityfile_delete, {}, 'gentityfile_delete'),

    (r'^gentitygenericdata/(?P<gg_id>\d+)/download/$',
     views.download_gentitygenericdata, {}, 'gentitygenericdata_dl'),
    (r'^gentitygenericdata/edit/(?P<ggenericdata_id>\d+)/$',
     views.gentitygenericdata_edit, {}, 'gentitygenericdata_edit'),
    (r'^gentitygenericdata/add/$', views.gentitygenericdata_add, {},
     'gentitygenericdata_add'),
    (r'^gentitygenericdata/delete/(?P<ggenericdata_id>\d+)/$',
     views.gentitygenericdata_delete, {}, 'gentitygenericdata_delete'),

    (r'^gentityevent/edit/(?P<gentityevent_id>\d+)/$',
     views.gentityevent_edit, {}, 'gentityevent_edit'),
    (r'^gentityevent/add/$', views.gentityevent_add, {},
     'gentityevent_add'),
    (r'^gentityevent/delete/(?P<gentityevent_id>\d+)/$',
     views.gentityevent_delete, {}, 'gentityevent_delete'),

    (r'^gentityaltcode/edit/(?P<gentityaltcode_id>\d+)/$',
     views.gentityaltcode_edit, {}, 'gentityaltcode_edit'),
    (r'^gentityaltcode/add/$', views.gentityaltcode_add, {},
     'gentityaltcode_add'),
    (r'^gentityaltcode/delete/(?P<gentityaltcode_id>\d+)/$',
     views.gentityaltcode_delete, {}, 'gentityaltcode_delete'),

    (r'^overseer/edit/(?P<overseer_id>\d+)/$',
     views.overseer_edit, {}, 'overseer_edit'),
    (r'^overseer/add/$', views.overseer_add, {}, 'overseer_add'),
    (r'^overseer/delete/(?P<overseer_id>\d+)/$',
     views.overseer_delete, {}, 'overseer_delete'),

    (r'^(?P<layer>[^/]+)/kml/$',
     views.StationListView.as_view(template_name='placemarks.kml')),

    (r'^bounding_box/$', views.BoundingBoxView.as_view(), {},
        'bounding_box'),

    url(r'^add/(?P<model_name>.+)/$', views.ModelAddView.as_view(),
        name='model_add'),
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('hcore', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='gpoint',
            name='approximate',
            field=models.BooleanField(default=False),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='instrument',
            name='is_active',
            field=models.BooleanField(default=False),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='overseer',
            name='is_current',
            field=models.BooleanField(default=False),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='station',
            name='is_active',
            field=models.BooleanField(default=False),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='station',
            name='is_automatic',
            field=models.BooleanField(default=False),
            preserve_default=True,
        ),
    ]



from django.contrib.auth import REDIRECT_FIELD_NAME
from django.contrib.auth.decorators import user_passes_test
from django.db.models import Q
from django.contrib.gis.geos import Polygon
from django.http import Http404
from django.conf import settings
from django.core.exceptions import FieldError

from enhydris.hcore.models import *


def timeseries_permission(function=None, redirect_field_name=REDIRECT_FIELD_NAME):
    """
    Decorator for timeseries_download view that checks if only authenticated
    users have access to the data and then acts like the login_required
    decorator. Otherwise, it just calls the function.
    """

    if settings.ENHYDRIS_TSDATA_AVAILABLE_FOR_ANONYMOUS_USERS:
        return function

    actual_decorator = user_passes_test(
        lambda u: u.is_authenticated(),
        redirect_field_name=redirect_field_name
    )

    if function:
        return actual_decorator(function)
    return actual_decorator


def gentityfile_permission(function=None, redirect_field_name=REDIRECT_FIELD_NAME):
    """
    Decorator for gentityfile_download view that checks if only authenticated
    users have access to the data and then acts like the login_required
    decorator. Otherwise, it just calls the function.
    """

    if settings.ENHYDRIS_TSDATA_AVAILABLE_FOR_ANONYMOUS_USERS:
        return function

    actual_decorator = user_passes_test(
        lambda u: u.is_authenticated(),
        redirect_field_name=redirect_field_name
    )

    if function:
        return actual_decorator(function)
    return actual_decorator



import os

from django import forms, db
from django.conf import settings
from django.contrib.gis.geos import Point
from django.forms import ModelForm
from django.utils.translation import ugettext_lazy as _

from ajax_select.fields import AutoCompleteSelectMultipleField
from captcha.fields import CaptchaField
from registration.forms import RegistrationFormTermsOfService

from pthelma import timeseries
from enhydris.hcore.widgets import SelectWithPop
from enhydris.hcore.models import (
    Station, Instrument, Person, Overseer, FileType, GentityFile,
    GentityGenericDataType, GentityGenericData, GentityAltCodeType,
    GentityAltCode, EventType, GentityEvent, Gentity, Gpoint,
    PoliticalDivision, WaterBasin, WaterDivision, Lentity, InstrumentType,
    TimeStep, Variable, UnitOfMeasurement, TimeZone, IntervalType, Timeseries)


class OverseerForm(ModelForm):

    station_objects = Station.objects.all()
    if len(settings.ENHYDRIS_SITE_STATION_FILTER) > 0:
        station_objects = station_objects.filter(
            **settings.ENHYDRIS_SITE_STATION_FILTER)
    station = forms.ModelChoiceField(station_objects, label='Station',
                                     empty_label=None)
    person = forms.ModelChoiceField(Person.objects,
                                    widget=SelectWithPop(model_name='person'))

    class Meta:
        model = Overseer
        exclude = []

    def __init__(self, *args, **kwargs):
        user = kwargs.pop('user', None)
        gentity_id = kwargs.pop('gentity_id', None)
        super(OverseerForm, self).__init__(*args, **kwargs)

        if user and not user.is_superuser:
            perms = user.get_rows_with_permission(Station(), 'edit')
            ids = [p.object_id for p in perms]
            self.fields["station"].queryset = Station.objects.filter(
                id__in=ids)
        if gentity_id:
            self.fields["station"].queryset = Station.objects.filter(
                id=gentity_id)


class GentityFileForm(ModelForm):

    station_objects = Station.objects.all()
    if len(settings.ENHYDRIS_SITE_STATION_FILTER) > 0:
        station_objects = station_objects.filter(
            **settings.ENHYDRIS_SITE_STATION_FILTER)
    gentity = forms.ModelChoiceField(station_objects, label='Station',
                                     empty_label=None)
    file_type = forms.ModelChoiceField(
        FileType.objects, widget=SelectWithPop(model_name='filetype'))

    class Meta:
        model = GentityFile
        exclude = []

    def __init__(self, *args, **kwargs):
        user = kwargs.pop('user', None)
        gentity_id = kwargs.pop('gentity_id', None)
        super(GentityFileForm, self).__init__(*args, **kwargs)

        if user and not user.is_superuser:
            perms = user.get_rows_with_permission(Station(), 'edit')
            ids = [p.object_id for p in perms]
            self.fields["gentity"].queryset = Station.objects.filter(
                id__in=ids)
        if gentity_id:
            self.fields["gentity"].queryset = Station.objects.filter(
                id=gentity_id)


class GentityGenericDataForm(ModelForm):

    station_objects = Station.objects.all()
    if len(settings.ENHYDRIS_SITE_STATION_FILTER) > 0:
        station_objects = station_objects.filter(
            **settings.ENHYDRIS_SITE_STATION_FILTER)
    gentity = forms.ModelChoiceField(station_objects, label='Station',
                                     empty_label=None)
    data_type = forms.ModelChoiceField(
        GentityGenericDataType.objects,
        label='Data type',
        widget=SelectWithPop(model_name='gentitygenericdatatype'))

    class Meta:
        model = GentityGenericData
        exclude = []

    def __init__(self, *args, **kwargs):
        user = kwargs.pop('user', None)
        gentity_id = kwargs.pop('gentity_id', None)
        super(GentityGenericDataForm, self).__init__(*args, **kwargs)

        if user and not user.is_superuser:
            perms = user.get_rows_with_permission(Station(), 'edit')
            ids = [p.object_id for p in perms]
            self.fields["gentity"].queryset = Station.objects.filter(
                id__in=ids)
        if gentity_id:
            self.fields["gentity"].queryset = Station.objects.filter(
                id=gentity_id)


class GentityAltCodeForm(ModelForm):

    station_objects = Station.objects.all()
    if len(settings.ENHYDRIS_SITE_STATION_FILTER) > 0:
        station_objects = station_objects.filter(
            **settings.ENHYDRIS_SITE_STATION_FILTER)
    gentity = forms.ModelChoiceField(station_objects, label='Station',
                                     empty_label=None)
    type = forms.ModelChoiceField(
        GentityAltCodeType.objects,
        widget=SelectWithPop(model_name='gentityaltcodetype'))

    class Meta:
        model = GentityAltCode
        exclude = []

    def __init__(self, *args, **kwargs):
        user = kwargs.pop('user', None)
        gentity_id = kwargs.pop('gentity_id', None)
        super(GentityAltCodeForm, self).__init__(*args, **kwargs)

        if user and not user.is_superuser:
            perms = user.get_rows_with_permission(Station(), 'edit')
            ids = [p.object_id for p in perms]
            self.fields["gentity"].queryset = Station.objects.filter(
                id__in=ids)
        if gentity_id:
            self.fields["gentity"].queryset = Station.objects.filter(
                id=gentity_id)


class GentityEventForm(ModelForm):

    station_objects = Station.objects.all()
    if len(settings.ENHYDRIS_SITE_STATION_FILTER) > 0:
        station_objects = station_objects.filter(
            **settings.ENHYDRIS_SITE_STATION_FILTER)
    gentity = forms.ModelChoiceField(station_objects, label='Station',
                                     empty_label=None)
    type = forms.ModelChoiceField(
        EventType.objects, widget=SelectWithPop(model_name='eventtype'))

    class Meta:
        model = GentityEvent
        exclude = []

    def __init__(self, *args, **kwargs):
        user = kwargs.pop('user', None)
        gentity_id = kwargs.pop('gentity_id', None)
        super(GentityEventForm, self).__init__(*args, **kwargs)

        if user and not user.is_superuser:
            perms = user.get_rows_with_permission(Station(), 'edit')
            ids = [p.object_id for p in perms]
            self.fields["gentity"].queryset = Station.objects.filter(
                id__in=ids)
        if gentity_id:
            self.fields["gentity"].queryset = Station.objects.filter(
                id=gentity_id)


class GentityForm(ModelForm):

    class Meta:
        model = Gentity
        exclude = []


class GpointForm(GentityForm):
    abscissa = forms.FloatField(required=False)
    ordinate = forms.FloatField(required=False)
    srid = forms.IntegerField(required=False)

    class Meta:
        model = Gpoint
        exclude = ('point',)

    def clean(self):
        cleaned_data = self.cleaned_data
        if bool('abscissa' in cleaned_data and cleaned_data['abscissa']) ^ \
                bool('ordinate' in cleaned_data and cleaned_data['ordinate']):
            raise forms.ValidationError(_("Both coordinates (abscissa, "
                                          "ordinate) should be "
                                          "provided. If position is "
                                          "undefined, leave both empty."))
        if ('abscissa' in cleaned_data and cleaned_data['abscissa']) and not \
                ('srid' in cleaned_data and cleaned_data['srid']):
            self._errors['srid'] =\
                self.error_class(["Since you have provided coordinates, "
                                  "a srid should be provided as well. "
                                  "If you don't know about srid settings, "
                                  "see http://en.wikipedia.org/wiki/Srid/ . "
                                  "If you wish to enter geographical "
                                  "coordinates in WGS-84, you may "
                                  "enter a value of 4326 for srid."])
        super(GpointForm, self).clean()
        return cleaned_data

    def save(self, commit=True, *args, **kwargs):

        gpoint = super(GpointForm, self).save(commit=False)
        abscissa = self.cleaned_data['abscissa']
        ordinate = self.cleaned_data['ordinate']
        srid = self.cleaned_data['srid']

        if (abscissa is None) or (ordinate is None):
            gpoint.point = None
        else:
            if srid is None:
                srid = 4326
            gpoint.point = Point(x=abscissa, y=ordinate, srid=srid)

        # This may not be a good idea !
        if commit:
            gpoint.save()

        return gpoint


class StationForm(GpointForm, GentityForm):
    """
    In this form, we overide the default fields with our own to allow inline
    creation of models that a Station has foreign keys to.

    To achieve this we use a custom widget which adds in each select box a 'Add
    New' button which takes care of the object creation and also updating the
    original select box with the new entries. The only caveat is that we need
    to pass manually to the widget the name of the foreign object as it is
    depicted in our database in cases where the field name is not the same.
    """
    class Meta:
        model = Station
        exclude = ('overseers', 'creator', 'point')

    political_division = forms.ModelChoiceField(
        PoliticalDivision.objects,
        widget=SelectWithPop(model_name='politicaldivision'),
        required=False)
    water_basin = forms.ModelChoiceField(
        WaterBasin.objects,
        widget=SelectWithPop(model_name='waterbasin'),
        required=False)
    water_division = forms.ModelChoiceField(
        WaterDivision.objects,
        widget=SelectWithPop(model_name='waterdivision'),
        required=False)
    # owner should be modified to allow either Person or Organization add
    owner = forms.ModelChoiceField(
        Lentity.objects, widget=SelectWithPop(model_name='lentity'))

    if settings.ENHYDRIS_USERS_CAN_ADD_CONTENT:
        maintainers = AutoCompleteSelectMultipleField(
            'maintainers', required=False)

    def clean_altitude(self):
        value = self.cleaned_data['altitude']
        if not value is None and (value > 8850 or value < -422):
            raise forms.ValidationError(
                _("%f is not a valid altitude") % (value,))
        return self.cleaned_data['altitude']


class InstrumentForm(ModelForm):

    station_objects = Station.objects.all()
    if len(settings.ENHYDRIS_SITE_STATION_FILTER) > 0:
        station_objects = station_objects.filter(
            **settings.ENHYDRIS_SITE_STATION_FILTER)
    station = forms.ModelChoiceField(station_objects, label='Stations',
                                     empty_label=None)
    type = forms.ModelChoiceField(
        InstrumentType.objects,
        widget=SelectWithPop(model_name='instrumenttype'))

    class Meta:
        model = Instrument
        exclude = []

    def __init__(self, *args, **kwargs):
        user = kwargs.pop('user', None)
        gentity_id = kwargs.pop('gentity_id', None)
        super(InstrumentForm, self).__init__(*args, **kwargs)

        if user and not user.is_superuser:
            perms = user.get_rows_with_permission(Station(), 'edit')
            ids = [p.object_id for p in perms]
            self.fields["station"].queryset = Station.objects.filter(
                id__in=ids)
        if gentity_id:
            self.fields["station"].queryset = Station.objects.filter(
                id=gentity_id)


def _int_xor(i1, i2):
    """Return True if one and only one of i1 and i2 is zero."""
    return (i1 or i2) and not (i1 and i2)


class TimeStepForm(ModelForm):

    class Meta:
        model = TimeStep
        exclude = []

    def clean(self):
        """
        This clean function ensures that length minutes and length months have
        valid values.
        """
        length_minutes = self.cleaned_data.get('length_minutes', None)
        length_months = self.cleaned_data.get('length_months', None)
        if not _int_xor(length_minutes, length_months):
            raise forms.ValidationError(
                _("Invalid timestep: exactly one of"
                  " minutes and months must be zero"))
        return self.cleaned_data


class TimeseriesForm(ModelForm):
    """
    This form extends the basic Timeseries model with a file field through
    which a user may upload additional data.
    """

    gentity = forms.ModelChoiceField(
        Gentity.objects.all(),
        empty_label=None,
        label='Station')
    instrument = forms.ModelChoiceField(
        Instrument.objects.all(),
        required=False,
        label='Instrument')
    variable = forms.ModelChoiceField(
        Variable.objects,
        widget=SelectWithPop(model_name='variable'))
    unit_of_measurement = forms.ModelChoiceField(
        UnitOfMeasurement.objects,
        widget=SelectWithPop(model_name='unitofmeasurement'))
    time_zone = forms.ModelChoiceField(
        TimeZone.objects,
        widget=SelectWithPop(model_name='timezone'))
    time_step = forms.ModelChoiceField(
        TimeStep.objects,
        widget=SelectWithPop(model_name='timestep'),
        required=False)
    interval_type = forms.ModelChoiceField(
        IntervalType.objects,
        required=False)

    class Meta:
        model = Timeseries
        exclude = []

    def __init__(self, *args, **kwargs):
        user = kwargs.pop('user', None)
        gentity_id = kwargs.pop('gentity_id', None)
        instrument_id = kwargs.pop('instrument_id', None)
        super(TimeseriesForm, self).__init__(*args, **kwargs)

        if user and not user.is_superuser:
            perms = user.get_rows_with_permission(Station(), 'edit')
            ids = [p.object_id for p in perms]
            self.fields["gentity"].queryset = Station.objects.filter(
                id__in=ids)
            perms = user.get_rows_with_permission(Instrument(), 'edit')
            ids = [p.object_id for p in perms]
            self.fields["instrument"].queryset = Instrument.objects.filter(
                id__in=ids)
        if gentity_id:
            self.fields["gentity"].queryset = Station.objects.filter(
                id=gentity_id)
            self.fields["instrument"].queryset = Instrument.objects.filter(
                station__id=gentity_id)
        if instrument_id:
            self.fields["instrument"].queryset = Instrument.objects.filter(
                id=instrument_id)
            self.fields["instrument"].empty_label = None

    def clean_data(self):
        # Check if file contains valid timeseries data.
        if ('data' not in self.cleaned_data) or not self.cleaned_data['data']:
            return None
        self.cleaned_data['data'].seek(0)
        ts = timeseries.Timeseries()
        data = self.cleaned_data['data']

        try:
            ts.read_file(self.cleaned_data['data'])
        except Exception as e:
            raise forms.ValidationError(str(e))

        return self.cleaned_data['data']

    def clean(self):
        """
        This function checks the timestep and offset values and reports
        inconsistencies.
        """
        time_step = self.cleaned_data.get('time_step', None)
        nominal_offset_minutes = self.cleaned_data.get(
            'nominal_offset_minutes', None)
        nominal_offset_months = self.cleaned_data.get(
            'nominal_offset_months', None)
        actual_offset_minutes = self.cleaned_data.get(
            'actual_offset_minutes', None)
        actual_offset_months = self.cleaned_data.get(
            'actual_offset_months', None)

        if not time_step:
            if nominal_offset_minutes or nominal_offset_months or \
                    actual_offset_minutes or actual_offset_months:
                raise forms.ValidationError(
                    _("Invalid Timestep: If time step is"
                      " null, the offsets must also be null!"))
        else:
            if actual_offset_minutes is None or actual_offset_months is None:
                raise forms.ValidationError(
                    _("Invalid offset: If time step is"
                      " not null, actual offset values must be provided!"))
            if (nominal_offset_minutes is None
                and nominal_offset_months is not None) \
                    or (nominal_offset_minutes is not None
                        and nominal_offset_months is None):
                raise forms.ValidationError(
                    _("Invalid offsets: Nominal"
                      " offsets must be both null or both not null!"))

        #add a validation test for instrument in station:
        instr = self.cleaned_data.get('instrument', None)
        if instr:
            stat = self.cleaned_data.get('gentity', None)
            assert(stat)
            if Instrument.objects.filter(id=instr.id,
                                         station__id=stat.id).count() < 1:
                raise forms.ValidationError(_("Selected instrument "
                                              "not in selected station"))

        # XXX: This is not a good idea but it's the only way to handle append
        # errors. We save the data in the clean function instead of the save
        # and if an error occurs we show the error in the form. Another way
        # would be to write a try_append() function in pthelma.timeseries but
        # this is specific to enhydris and should not be part of the pthelma
        # module.

        if 'data' in self.cleaned_data and self.cleaned_data['data']:
            data = self.cleaned_data['data']
            ts = timeseries.Timeseries(int(self.instance.id))
            data.seek(0)
            ts.read_file(data)
            if self.cleaned_data['data_policy'] == 'A':
                try:
                    ts.append_to_db(db.connection, commit=False)
                except Exception, e:
                    raise forms.ValidationError(_(e.message))

        return self.cleaned_data

    def save(self, commit=True, *args, **kwargs):

        tseries = super(TimeseriesForm, self).save(commit=False)

        # This may not be a good idea !
        if commit:
            tseries.save()

        # Handle pushing additional data if present back to the db
        if 'data' in self.cleaned_data and self.cleaned_data['data']:
            data = self.cleaned_data['data']
            ts = timeseries.Timeseries(int(self.instance.id))
            data.seek(0)
            ts.read_file(data)
            if self.cleaned_data['data_policy'] == 'A':
                pass
                # ts.append_to_db(db.connection, commit=False)
            else:
                afilename = os.path.join(settings.ENHYDRIS_TS_GRAPH_CACHE_DIR,
                                         '%d.hts' % int(self.instance.id))
                if os.path.exists(afilename):
                    os.remove(afilename)
                ts.write_to_db(db.connection, commit=False)

        return tseries


class TimeseriesDataForm(TimeseriesForm):
    """
    Additional timeseries form to present the data upload fields
    """
    data = forms.FileField(required=False)
    data_policy = forms.ChoiceField(
        label=_('New data policy'),
        required=False,
        choices=(('A', 'Append to existing'),
                    ('O', 'Overwrite existing'),))


class RegistrationForm(RegistrationFormTermsOfService):
    """
    Extension of the default registration form to include a captcha
    """
    captcha = CaptchaField()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import enhydris.hcore.models
import django.contrib.gis.db.models.fields
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('dbsync', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='EventType',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('original_id', models.IntegerField(null=True, editable=False, blank=True)),
                ('last_modified', models.DateTimeField(auto_now=True, null=True)),
                ('descr', models.CharField(max_length=200, blank=True)),
                ('descr_alt', models.CharField(max_length=200, blank=True)),
                ('original_db', models.ForeignKey(editable=False, to='dbsync.Database', null=True)),
            ],
            options={
                'ordering': ('descr',),
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='FileType',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('original_id', models.IntegerField(null=True, editable=False, blank=True)),
                ('last_modified', models.DateTimeField(auto_now=True, null=True)),
                ('descr', models.CharField(max_length=200, blank=True)),
                ('descr_alt', models.CharField(max_length=200, blank=True)),
                ('mime_type', models.CharField(max_length=64)),
                ('original_db', models.ForeignKey(editable=False, to='dbsync.Database', null=True)),
            ],
            options={
                'ordering': ('descr',),
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Gentity',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('original_id', models.IntegerField(null=True, editable=False, blank=True)),
                ('last_modified', models.DateTimeField(auto_now=True, null=True)),
                ('name', models.CharField(max_length=200, blank=True)),
                ('short_name', models.CharField(max_length=50, blank=True)),
                ('remarks', models.TextField(blank=True)),
                ('name_alt', models.CharField(max_length=200, blank=True)),
                ('short_name_alt', models.CharField(max_length=50, blank=True)),
                ('remarks_alt', models.TextField(blank=True)),
            ],
            options={
                'ordering': ('name',),
                'verbose_name_plural': 'Gentities',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Garea',
            fields=[
                ('gentity_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='hcore.Gentity')),
                ('area', models.FloatField(null=True, blank=True)),
                ('mpoly', django.contrib.gis.db.models.fields.MultiPolygonField(srid=4326, null=True, blank=True)),
            ],
            options={
            },
            bases=('hcore.gentity',),
        ),
        migrations.CreateModel(
            name='GentityAltCode',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('original_id', models.IntegerField(null=True, editable=False, blank=True)),
                ('last_modified', models.DateTimeField(auto_now=True, null=True)),
                ('value', models.CharField(max_length=100)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='GentityAltCodeType',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('original_id', models.IntegerField(null=True, editable=False, blank=True)),
                ('last_modified', models.DateTimeField(auto_now=True, null=True)),
                ('descr', models.CharField(max_length=200, blank=True)),
                ('descr_alt', models.CharField(max_length=200, blank=True)),
                ('original_db', models.ForeignKey(editable=False, to='dbsync.Database', null=True)),
            ],
            options={
                'ordering': ('descr',),
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='GentityEvent',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('original_id', models.IntegerField(null=True, editable=False, blank=True)),
                ('last_modified', models.DateTimeField(auto_now=True, null=True)),
                ('date', models.DateField()),
                ('user', models.CharField(max_length=64)),
                ('report', models.TextField(blank=True)),
                ('report_alt', models.TextField(blank=True)),
            ],
            options={
                'ordering': ['-date'],
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='GentityFile',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('original_id', models.IntegerField(null=True, editable=False, blank=True)),
                ('last_modified', models.DateTimeField(auto_now=True, null=True)),
                ('date', models.DateField(null=True, blank=True)),
                ('content', models.FileField(upload_to=b'gentityfile')),
                ('descr', models.CharField(max_length=100)),
                ('remarks', models.TextField(blank=True)),
                ('descr_alt', models.CharField(max_length=100)),
                ('remarks_alt', models.TextField(blank=True)),
                ('file_type', models.ForeignKey(to='hcore.FileType')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='GentityGenericData',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('original_id', models.IntegerField(null=True, editable=False, blank=True)),
                ('last_modified', models.DateTimeField(auto_now=True, null=True)),
                ('descr', models.CharField(max_length=100)),
                ('remarks', models.TextField(blank=True)),
                ('descr_alt', models.CharField(max_length=100)),
                ('remarks_alt', models.TextField(blank=True)),
                ('content', models.TextField(blank=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='GentityGenericDataType',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('original_id', models.IntegerField(null=True, editable=False, blank=True)),
                ('last_modified', models.DateTimeField(auto_now=True, null=True)),
                ('descr', models.CharField(max_length=200, blank=True)),
                ('descr_alt', models.CharField(max_length=200, blank=True)),
                ('file_extension', models.CharField(max_length=16)),
                ('original_db', models.ForeignKey(editable=False, to='dbsync.Database', null=True)),
            ],
            options={
                'ordering': ('descr',),
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Gline',
            fields=[
                ('gentity_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='hcore.Gentity')),
                ('length', models.FloatField(null=True, blank=True)),
                ('linestring', django.contrib.gis.db.models.fields.LineStringField(srid=4326, null=True, blank=True)),
            ],
            options={
            },
            bases=('hcore.gentity',),
        ),
        migrations.CreateModel(
            name='Gpoint',
            fields=[
                ('gentity_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='hcore.Gentity')),
                ('srid', models.IntegerField(null=True, blank=True)),
                ('approximate', models.BooleanField()),
                ('altitude', models.FloatField(null=True, blank=True)),
                ('asrid', models.IntegerField(null=True, blank=True)),
                ('point', django.contrib.gis.db.models.fields.PointField(srid=4326, null=True, blank=True)),
            ],
            options={
            },
            bases=('hcore.gentity',),
        ),
        migrations.CreateModel(
            name='Instrument',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('original_id', models.IntegerField(null=True, editable=False, blank=True)),
                ('last_modified', models.DateTimeField(auto_now=True, null=True)),
                ('manufacturer', models.CharField(max_length=50, blank=True)),
                ('model', models.CharField(max_length=50, blank=True)),
                ('is_active', models.BooleanField()),
                ('start_date', models.DateField(null=True, blank=True)),
                ('end_date', models.DateField(null=True, blank=True)),
                ('name', models.CharField(max_length=100, blank=True)),
                ('remarks', models.TextField(blank=True)),
                ('name_alt', models.CharField(max_length=100, blank=True)),
                ('remarks_alt', models.TextField(blank=True)),
                ('original_db', models.ForeignKey(editable=False, to='dbsync.Database', null=True)),
            ],
            options={
                'ordering': ('name',),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='InstrumentType',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('original_id', models.IntegerField(null=True, editable=False, blank=True)),
                ('last_modified', models.DateTimeField(auto_now=True, null=True)),
                ('descr', models.CharField(max_length=200, blank=True)),
                ('descr_alt', models.CharField(max_length=200, blank=True)),
                ('original_db', models.ForeignKey(editable=False, to='dbsync.Database', null=True)),
            ],
            options={
                'ordering': ('descr',),
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='IntervalType',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('original_id', models.IntegerField(null=True, editable=False, blank=True)),
                ('last_modified', models.DateTimeField(auto_now=True, null=True)),
                ('descr', models.CharField(max_length=200, blank=True)),
                ('descr_alt', models.CharField(max_length=200, blank=True)),
                ('value', models.CharField(max_length=50)),
                ('original_db', models.ForeignKey(editable=False, to='dbsync.Database', null=True)),
            ],
            options={
                'ordering': ('descr',),
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Lentity',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('original_id', models.IntegerField(null=True, editable=False, blank=True)),
                ('last_modified', models.DateTimeField(auto_now=True, null=True)),
                ('remarks', models.TextField(blank=True)),
                ('remarks_alt', models.TextField(blank=True)),
                ('ordering_string', models.CharField(max_length=255, null=True, editable=False, blank=True)),
            ],
            options={
                'ordering': ('ordering_string',),
                'verbose_name_plural': 'Lentities',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Organization',
            fields=[
                ('lentity_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='hcore.Lentity')),
                ('name', models.CharField(max_length=200, blank=True)),
                ('acronym', models.CharField(max_length=50, blank=True)),
                ('name_alt', models.CharField(max_length=200, blank=True)),
                ('acronym_alt', models.CharField(max_length=50, blank=True)),
            ],
            options={
                'ordering': ('name',),
            },
            bases=('hcore.lentity',),
        ),
        migrations.CreateModel(
            name='Overseer',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('original_id', models.IntegerField(null=True, editable=False, blank=True)),
                ('last_modified', models.DateTimeField(auto_now=True, null=True)),
                ('is_current', models.BooleanField()),
                ('start_date', models.DateField(null=True, blank=True)),
                ('end_date', models.DateField(null=True, blank=True)),
                ('original_db', models.ForeignKey(editable=False, to='dbsync.Database', null=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Person',
            fields=[
                ('lentity_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='hcore.Lentity')),
                ('last_name', models.CharField(max_length=100, blank=True)),
                ('first_name', models.CharField(max_length=100, blank=True)),
                ('middle_names', models.CharField(max_length=100, blank=True)),
                ('initials', models.CharField(max_length=20, blank=True)),
                ('last_name_alt', models.CharField(max_length=100, blank=True)),
                ('first_name_alt', models.CharField(max_length=100, blank=True)),
                ('middle_names_alt', models.CharField(max_length=100, blank=True)),
                ('initials_alt', models.CharField(max_length=20, blank=True)),
            ],
            options={
                'ordering': ('last_name', 'first_name'),
            },
            bases=('hcore.lentity',),
        ),
        migrations.CreateModel(
            name='PoliticalDivision',
            fields=[
                ('garea_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='hcore.Garea')),
                ('code', models.CharField(max_length=5, blank=True)),
                ('parent', models.ForeignKey(blank=True, to='hcore.PoliticalDivision', null=True)),
            ],
            options={
            },
            bases=('hcore.garea',),
        ),
        migrations.CreateModel(
            name='Station',
            fields=[
                ('gpoint_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='hcore.Gpoint')),
                ('is_automatic', models.BooleanField()),
                ('is_active', models.BooleanField()),
                ('start_date', models.DateField(null=True, blank=True)),
                ('end_date', models.DateField(null=True, blank=True)),
                ('copyright_holder', models.TextField()),
                ('copyright_years', models.CharField(max_length=10)),
                ('creator', models.ForeignKey(related_name='created_stations', blank=True, to=settings.AUTH_USER_MODEL, null=True)),
                ('maintainers', models.ManyToManyField(related_name='maintaining_stations', null=True, to=settings.AUTH_USER_MODEL, blank=True)),
                ('overseers', models.ManyToManyField(related_name='stations_overseen', through='hcore.Overseer', to='hcore.Person')),
                ('owner', models.ForeignKey(related_name='owned_stations', to='hcore.Lentity')),
            ],
            options={
            },
            bases=('hcore.gpoint',),
        ),
        migrations.CreateModel(
            name='StationType',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('original_id', models.IntegerField(null=True, editable=False, blank=True)),
                ('last_modified', models.DateTimeField(auto_now=True, null=True)),
                ('descr', models.CharField(max_length=200, blank=True)),
                ('descr_alt', models.CharField(max_length=200, blank=True)),
                ('original_db', models.ForeignKey(editable=False, to='dbsync.Database', null=True)),
            ],
            options={
                'ordering': ('descr',),
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Timeseries',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('original_id', models.IntegerField(null=True, editable=False, blank=True)),
                ('last_modified', models.DateTimeField(auto_now=True, null=True)),
                ('name', models.CharField(max_length=200, blank=True)),
                ('name_alt', models.CharField(default=b'', max_length=200, blank=True)),
                ('hidden', models.BooleanField(default=False)),
                ('precision', models.SmallIntegerField(null=True, blank=True)),
                ('remarks', models.TextField(blank=True)),
                ('remarks_alt', models.TextField(default=b'', blank=True)),
                ('nominal_offset_minutes', models.PositiveIntegerField(null=True, blank=True)),
                ('nominal_offset_months', models.PositiveSmallIntegerField(null=True, blank=True)),
                ('actual_offset_minutes', models.IntegerField(null=True, blank=True)),
                ('actual_offset_months', models.SmallIntegerField(null=True, blank=True)),
            ],
            options={
                'ordering': ('hidden',),
                'verbose_name': 'Time Series',
                'verbose_name_plural': 'Time Series',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='TimeStep',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('original_id', models.IntegerField(null=True, editable=False, blank=True)),
                ('last_modified', models.DateTimeField(auto_now=True, null=True)),
                ('descr', models.CharField(max_length=200, blank=True)),
                ('descr_alt', models.CharField(max_length=200, blank=True)),
                ('length_minutes', models.PositiveIntegerField()),
                ('length_months', models.PositiveSmallIntegerField()),
                ('original_db', models.ForeignKey(editable=False, to='dbsync.Database', null=True)),
            ],
            options={
                'ordering': ('descr',),
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='TimeZone',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('original_id', models.IntegerField(null=True, editable=False, blank=True)),
                ('last_modified', models.DateTimeField(auto_now=True, null=True)),
                ('code', models.CharField(max_length=50)),
                ('utc_offset', models.SmallIntegerField()),
                ('original_db', models.ForeignKey(editable=False, to='dbsync.Database', null=True)),
            ],
            options={
                'ordering': ('utc_offset',),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='TsRecords',
            fields=[
                ('id', models.ForeignKey(primary_key=True, db_column=b'id', serialize=False, to='hcore.Timeseries')),
                ('top', models.TextField(blank=True)),
                ('middle', enhydris.hcore.models.BlobField(null=True, blank=True)),
                ('bottom', models.TextField()),
            ],
            options={
                'db_table': 'ts_records',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='UnitOfMeasurement',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('original_id', models.IntegerField(null=True, editable=False, blank=True)),
                ('last_modified', models.DateTimeField(auto_now=True, null=True)),
                ('descr', models.CharField(max_length=200, blank=True)),
                ('descr_alt', models.CharField(max_length=200, blank=True)),
                ('symbol', models.CharField(max_length=50)),
                ('original_db', models.ForeignKey(editable=False, to='dbsync.Database', null=True)),
            ],
            options={
                'ordering': ('descr',),
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='UserProfile',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('fname', models.CharField(max_length=30, null=True, verbose_name='First Name', blank=True)),
                ('lname', models.CharField(max_length=30, null=True, verbose_name='Last Name', blank=True)),
                ('address', models.CharField(max_length=100, null=True, verbose_name='Location', blank=True)),
                ('organization', models.CharField(max_length=100, null=True, verbose_name='Organization', blank=True)),
                ('email_is_public', models.BooleanField(default=False)),
                ('user', models.ForeignKey(verbose_name='Username', to=settings.AUTH_USER_MODEL, unique=True)),
            ],
            options={
                'verbose_name': 'Profile',
                'verbose_name_plural': 'Profiles',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Variable',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('original_id', models.IntegerField(null=True, editable=False, blank=True)),
                ('last_modified', models.DateTimeField(auto_now=True, null=True)),
                ('descr', models.CharField(max_length=200, blank=True)),
                ('descr_alt', models.CharField(max_length=200, blank=True)),
                ('original_db', models.ForeignKey(editable=False, to='dbsync.Database', null=True)),
            ],
            options={
                'ordering': ('descr',),
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='WaterBasin',
            fields=[
                ('garea_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='hcore.Garea')),
                ('parent', models.ForeignKey(blank=True, to='hcore.WaterBasin', null=True)),
            ],
            options={
            },
            bases=('hcore.garea',),
        ),
        migrations.CreateModel(
            name='WaterDivision',
            fields=[
                ('garea_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='hcore.Garea')),
            ],
            options={
            },
            bases=('hcore.garea',),
        ),
        migrations.AddField(
            model_name='waterbasin',
            name='water_division',
            field=models.ForeignKey(blank=True, to='hcore.WaterDivision', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='unitofmeasurement',
            name='variables',
            field=models.ManyToManyField(to='hcore.Variable'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='timeseries',
            name='gentity',
            field=models.ForeignKey(related_name='timeseries', to='hcore.Gentity'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='timeseries',
            name='instrument',
            field=models.ForeignKey(blank=True, to='hcore.Instrument', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='timeseries',
            name='interval_type',
            field=models.ForeignKey(blank=True, to='hcore.IntervalType', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='timeseries',
            name='original_db',
            field=models.ForeignKey(editable=False, to='dbsync.Database', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='timeseries',
            name='time_step',
            field=models.ForeignKey(blank=True, to='hcore.TimeStep', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='timeseries',
            name='time_zone',
            field=models.ForeignKey(to='hcore.TimeZone'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='timeseries',
            name='unit_of_measurement',
            field=models.ForeignKey(to='hcore.UnitOfMeasurement'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='timeseries',
            name='variable',
            field=models.ForeignKey(to='hcore.Variable'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='station',
            name='stype',
            field=models.ManyToManyField(to='hcore.StationType', verbose_name=b'type'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='overseer',
            name='person',
            field=models.ForeignKey(to='hcore.Person'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='overseer',
            name='station',
            field=models.ForeignKey(to='hcore.Station'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='lentity',
            name='original_db',
            field=models.ForeignKey(editable=False, to='dbsync.Database', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='instrument',
            name='station',
            field=models.ForeignKey(to='hcore.Station'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='instrument',
            name='type',
            field=models.ForeignKey(to='hcore.InstrumentType'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='gline',
            name='gpoint1',
            field=models.ForeignKey(related_name='glines1', blank=True, to='hcore.Gpoint', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='gline',
            name='gpoint2',
            field=models.ForeignKey(related_name='glines2', blank=True, to='hcore.Gpoint', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='gentitygenericdata',
            name='data_type',
            field=models.ForeignKey(to='hcore.GentityGenericDataType'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='gentitygenericdata',
            name='gentity',
            field=models.ForeignKey(to='hcore.Gentity'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='gentitygenericdata',
            name='original_db',
            field=models.ForeignKey(editable=False, to='dbsync.Database', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='gentityfile',
            name='gentity',
            field=models.ForeignKey(to='hcore.Gentity'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='gentityfile',
            name='original_db',
            field=models.ForeignKey(editable=False, to='dbsync.Database', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='gentityevent',
            name='gentity',
            field=models.ForeignKey(to='hcore.Gentity'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='gentityevent',
            name='original_db',
            field=models.ForeignKey(editable=False, to='dbsync.Database', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='gentityevent',
            name='type',
            field=models.ForeignKey(to='hcore.EventType'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='gentityaltcode',
            name='gentity',
            field=models.ForeignKey(to='hcore.Gentity'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='gentityaltcode',
            name='original_db',
            field=models.ForeignKey(editable=False, to='dbsync.Database', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='gentityaltcode',
            name='type',
            field=models.ForeignKey(to='hcore.GentityAltCodeType'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='gentity',
            name='original_db',
            field=models.ForeignKey(editable=False, to='dbsync.Database', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='gentity',
            name='political_division',
            field=models.ForeignKey(blank=True, to='hcore.PoliticalDivision', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='gentity',
            name='water_basin',
            field=models.ForeignKey(blank=True, to='hcore.WaterBasin', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='gentity',
            name='water_division',
            field=models.ForeignKey(blank=True, to='hcore.WaterDivision', null=True),
            preserve_default=True,
        ),
    ]



from django.db import models
from django.utils.translation import ugettext_lazy as _

class Database(models.Model):
    """
    This class holds information for the database synchronization. Each db in
    the sync pool must have a corresponding entry here.
    """
    name = models.CharField(max_length=64, blank=True)
    ip_address = models.CharField(_('IP address'), max_length=16)
    hostname = models.CharField(_('Hostname'),unique=True, max_length=255)
    descr = models.TextField(_('Description'),max_length=255, blank=True)
    last_sync = models.DateTimeField(null=True, editable=False)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


def move_water_division_from_water_basin(apps, schema_editor):
    """
    There is a duplicate field: Gentity.water_division and
    WaterBasin.water_division. Before removing the latter, we will move its
    contents, if any, to the former, provided it does not conflict with
    anything already there.
    """
    WaterBasin = apps.get_model('hcore', 'WaterBasin')
    Gentity = apps.get_model('hcore', 'Gentity')
    for basin in WaterBasin.objects.all():
        if basin.water_division is None:
            continue
        gentity = Gentity.objects.get(pk=basin.pk)
        if gentity.water_division is None:
            gentity.water_division = basin.water_division
            gentity.save()
            continue
        if gentity.water_division.id != basin.water_division.id:
            raise Exception(
                "The water basin with id={} has a different water division "
                "in the waterbasin table and in the gentity table. Please "
                "fix this issue manually and retry the migration.")


def dummy(apps, schema_editor):
    """Do nothing for reverse data migration."""
    pass


class Migration(migrations.Migration):

    dependencies = [
        ('hcore', '0002_auto_fix_booleans'),
    ]

    operations = [
        migrations.RunPython(move_water_division_from_water_basin, dummy),
        migrations.RemoveField(
            model_name='waterbasin',
            name='water_division',
        ),
    ]



# coding=UTF-8

from django import template
from django.utils import html

register = template.Library()


@register.simple_tag
def sorter(column, sort_order, label):
    """Return a label with a sorting indicator that can be used for sorting.

    {% sorter column_name sort_order label %}

    This tag returns "label", possibly followed by a downward or an upward
    arrow that indicates the current sorting method, all that enclosed in a
    href hyperlink that would cause the sorting order to be changed so that
    it goes primarily by the specified column. "sort_order" is the current
    sort order. If the first column of sort_order is the same as "column_name",
    the tag adds a downward or upward arrow (depending on whether the sorting
    is direct or reverse).
    """
    sign = ''
    indicator = ''
    if sort_order and (sort_order[0] == column):
        indicator = u'&nbsp;↓'
        sign = '-'
    elif sort_order and (sort_order[0] == '-' + column):
        indicator = u'&nbsp;↑'
    target = '?sort={}{}&{}'.format(
        sign, column, '&'.join(['sort=' + x for x in sort_order]))
    return '<a href="%s">%s%s</a>' % (html.escape(target), label, indicator)



from django.db import models
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.auth.models import User, Group


class Permission(models.Model):
    name = models.CharField(max_length=16)
    content_type = models.ForeignKey(ContentType,
                                     related_name="row_permissions")
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey('content_type', 'object_id')
    user = models.ForeignKey(User, null=True, blank=True)
    group = models.ForeignKey(Group, null=True, blank=True)

    class Meta:
        verbose_name = 'permission'
        verbose_name_plural = 'permissions'



from django.contrib import admin
from enhydris.dbsync.models import Database

class DatabaseAdmin(admin.ModelAdmin):
    list_display = [f.name for f in Database._meta.fields]

admin.site.register(Database, DatabaseAdmin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.db.models import Q


def check_empty(apps, schema_editor):
    """Check that the dbsync fields are empty before removing them."""
    model_names = ('EventType', 'FileType', 'Gentity', 'GentityAltCode',
                   'GentityAltCodeType', 'GentityEvent', 'GentityFile',
                   'GentityGenericData', 'GentityGenericDataType',
                   'Instrument', 'InstrumentType', 'IntervalType',
                   'Lentity', 'Overseer', 'StationType', 'Timeseries',
                   'TimeStep', 'TimeZone', 'UnitOfMeasurement', 'Variable')
    for model_name in model_names:
        model = apps.get_model('hcore', model_name)
        offending_rows = model.objects.filter(
            Q(original_db__isnull=False) | Q(original_id__isnull=False))
        if offending_rows.count():
            raise Exception(
                "One of the rows of the database table that corresponds "
                "to {} has original_id or original_db_id that is not null. "
                "The database cannot be updated, as it apparently has "
                "been using the dbsync features, which have been abolished "
                "in this version of Enhydris. Please use Enhydris version "
                "0.2.".format(model_name))


def dummy(apps, schema_editor):
    """Do nothing for reverse data migration."""
    pass


class Migration(migrations.Migration):

    dependencies = [
        ('hcore', '0004_auto_minor_related_field_fixes'),
    ]

    operations = [
        migrations.RunPython(check_empty, dummy),
        migrations.RemoveField(
            model_name='eventtype',
            name='original_db',
        ),
        migrations.RemoveField(
            model_name='eventtype',
            name='original_id',
        ),
        migrations.RemoveField(
            model_name='filetype',
            name='original_db',
        ),
        migrations.RemoveField(
            model_name='filetype',
            name='original_id',
        ),
        migrations.RemoveField(
            model_name='gentity',
            name='original_db',
        ),
        migrations.RemoveField(
            model_name='gentity',
            name='original_id',
        ),
        migrations.RemoveField(
            model_name='gentityaltcode',
            name='original_db',
        ),
        migrations.RemoveField(
            model_name='gentityaltcode',
            name='original_id',
        ),
        migrations.RemoveField(
            model_name='gentityaltcodetype',
            name='original_db',
        ),
        migrations.RemoveField(
            model_name='gentityaltcodetype',
            name='original_id',
        ),
        migrations.RemoveField(
            model_name='gentityevent',
            name='original_db',
        ),
        migrations.RemoveField(
            model_name='gentityevent',
            name='original_id',
        ),
        migrations.RemoveField(
            model_name='gentityfile',
            name='original_db',
        ),
        migrations.RemoveField(
            model_name='gentityfile',
            name='original_id',
        ),
        migrations.RemoveField(
            model_name='gentitygenericdata',
            name='original_db',
        ),
        migrations.RemoveField(
            model_name='gentitygenericdata',
            name='original_id',
        ),
        migrations.RemoveField(
            model_name='gentitygenericdatatype',
            name='original_db',
        ),
        migrations.RemoveField(
            model_name='gentitygenericdatatype',
            name='original_id',
        ),
        migrations.RemoveField(
            model_name='instrument',
            name='original_db',
        ),
        migrations.RemoveField(
            model_name='instrument',
            name='original_id',
        ),
        migrations.RemoveField(
            model_name='instrumenttype',
            name='original_db',
        ),
        migrations.RemoveField(
            model_name='instrumenttype',
            name='original_id',
        ),
        migrations.RemoveField(
            model_name='intervaltype',
            name='original_db',
        ),
        migrations.RemoveField(
            model_name='intervaltype',
            name='original_id',
        ),
        migrations.RemoveField(
            model_name='lentity',
            name='original_db',
        ),
        migrations.RemoveField(
            model_name='lentity',
            name='original_id',
        ),
        migrations.RemoveField(
            model_name='overseer',
            name='original_db',
        ),
        migrations.RemoveField(
            model_name='overseer',
            name='original_id',
        ),
        migrations.RemoveField(
            model_name='stationtype',
            name='original_db',
        ),
        migrations.RemoveField(
            model_name='stationtype',
            name='original_id',
        ),
        migrations.RemoveField(
            model_name='timeseries',
            name='original_db',
        ),
        migrations.RemoveField(
            model_name='timeseries',
            name='original_id',
        ),
        migrations.RemoveField(
            model_name='timestep',
            name='original_db',
        ),
        migrations.RemoveField(
            model_name='timestep',
            name='original_id',
        ),
        migrations.RemoveField(
            model_name='timezone',
            name='original_db',
        ),
        migrations.RemoveField(
            model_name='timezone',
            name='original_id',
        ),
        migrations.RemoveField(
            model_name='unitofmeasurement',
            name='original_db',
        ),
        migrations.RemoveField(
            model_name='unitofmeasurement',
            name='original_id',
        ),
        migrations.RemoveField(
            model_name='variable',
            name='original_db',
        ),
        migrations.RemoveField(
            model_name='variable',
            name='original_id',
        ),
    ]



from django.conf.urls import url, include, patterns
from django.contrib import admin
from django.contrib.auth.views import password_reset, password_reset_done
from django.conf import settings

from registration.backends.default.views import RegistrationView

from enhydris.hcore.forms import RegistrationForm
from enhydris.hcore.views import ProfileDetailView, ProfileEditView

admin.autodiscover()

urlpatterns = patterns(
    '',

    # Registration and profiles
    (r'^accounts/password/reset/$', password_reset,
     {'template_name': 'registration/password_reset.html'}, 'password_reset'),
    (r'^accounts/password/reset/done/$', password_reset_done,
     {'template_name': 'registration/password_reset_done.html'},
     'password_reset_done'),
    (r'^accounts/', include('registration.backends.default.urls')),

    url(r'^profile/$', ProfileDetailView.as_view(),
        name='current_user_profile'),
    url(r'^profile/edit/$',
        ProfileEditView.as_view(),
        name='profile_edit'),
    url(r'^profile/(?P<slug>[^/]+)/$', ProfileDetailView.as_view(),
        name='profile_detail'),

    (r'^i18n/', include('django.conf.urls.i18n')),
    # Uncomment the next line to enable the admin:
    (r'^admin/', include(admin.site.urls)),
    (r'^ajax/', include('ajax_select.urls')),
    (r'^api/', include('enhydris.api.urls')),
    (r'^captcha/', include('captcha.urls')),
    (r'', include('enhydris.hcore.urls')),
)

if getattr(settings, 'REGISTRATION_OPEN', True):
    urlpatterns = patterns(
        '',
        url(r'^accounts/register/$',
            RegistrationView.as_view(form_class=RegistrationForm),
            name='registration_register')) + urlpatterns



from django.contrib.auth.models import User, Group
from django.contrib.contenttypes.models import ContentType
from django.db.models import Q
import new
import inspect

class MetaClass(type):
    """General metaclass which extends User/Group creators with additional
    methods"""
    def __new__(self, classname, classbases, classdict):
        try:
            frame = inspect.currentframe()
            frame = frame.f_back
            if frame.f_locals.has_key(classname):
                old_class = frame.f_locals.get(classname)
                for name,func in classdict.items():
                    if inspect.isfunction(func):
                        setattr(old_class, name, func)
                return old_class
            return type.__new__(self, classname, classbases, classdict)
        finally:
            del frame

class MetaObject(object):
    __metaclass__ = MetaClass

class User(MetaObject):
    """
    Extends class user with permission editing funtions.

    For example, for edit permission:

    $ user.has_row_perm(object, 'edit')
    False
    $ user.add_row_perm(object, 'edit')
    True
    $ user.has_row_perm(object, 'edit')
    True

    """
    def add_row_perm(self, instance, perm):
        """
        Add permission 'perm' to user 'self' for object(s) instance.
 
        instance variable may be an object or a queryset.
        """

        if type(instance).__name__ == 'QuerySet':
            for object in instance:
                from .models import Permission
                if self.has_row_perm(object, perm, True):
                    pass
                permission = Permission()
                permission.content_object = object
                permission.user = self
                permission.name = perm
                permission.save()
        else:

            if self.has_row_perm(instance, perm, True):
                return False
            permission = Permission()
            permission.content_object = instance
            permission.user = self
            permission.name = perm
            permission.save()

        return True

    def del_row_perm(self, instance, perm):
        """
        Remove permission 'perm' to user self for object instance.

        instance variable may be an object or a queryset.
        """

        if type(instance).__name__ == 'QuerySet':
            for object in instance:
                if not self.has_row_perm(object, perm, True):
                    pass
                content_type = ContentType.objects.get_for_model(object)
                perms = Permission.objects.filter(user=self,
                            content_type__pk=content_type.id,
                            object_id=object.id, name=perm)
                perms.delete()
        else:
            if not self.has_row_perm(instance, perm, True):
                return False
            content_type = ContentType.objects.get_for_model(instance)
            objects = Permission.objects.filter(user=self, content_type__pk=content_type.id, object_id=instance.id, name=perm)
            objects.delete()
        return True

    def has_row_perm(self, instance, perm, only_me=False):
        """
        Check if user has permission 'perm' on object instance.
        """
        if self.is_superuser:
            return True
        if not self.is_active:
            return False

        content_type = ContentType.objects.get_for_model(instance)
        objects = Permission.objects.filter(user=self, content_type__pk=content_type.id, object_id=instance.id, name=perm)
        if objects.count()>0:
            return True

        # check groups
        if not only_me:
            for group in self.groups.all():
                if group.has_row_perm(instance, perm):
                    return True
        return False

    def get_rows_with_permission(self, instance, perm):
        """
        Get all permission objects for this user on object 'instance' for perm
        'perm'

        NOTE: instance can be a model and then, this method returns all
        permissions for user 'self' on all objects of this model type.

        For example:

        $ from hcore.models import Station
        ...
        $ user.get_rows_with_permission(Station,'edit')

        this will return a list of all the stations the user has 'edit'
        permission.
        """
        content_type = ContentType.objects.get_for_model(instance)
        objects = Permission.objects.filter(Q(user=self) | Q(group__in=self.groups.all()), content_type__pk=content_type.id, name=perm)
        return objects


class Group(MetaObject):
    """Meta class to extend base Group class with permission editing methods"""
    def add_row_perm(self, instance, perm):

        if type(instance).__name__ == 'QuerySet':
            for object in instance:
                if self.has_row_perm(object, perm):
                    pass
                permission = Permission()
                permission.content_object = object
                permission.user = self
                permission.name = perm
                permission.save()
        else:
            if self.has_row_perm(instance, perm):
                return False
            permission = Permission()
            permission.content_object = instance
            permission.group = self
            permission.name = perm
            permission.save()
        return True

    def del_row_perm(self, instance, perm):

        if type(instance).__name__ == 'QuerySet':
            for object in instance:
                if not self.has_row_perm(object, perm, True):
                    pass
                content_type = ContentType.objects.get_for_model(object)
                perms = Permission.objects.filter(user=self, content_type__pk=content_type.id, object_id=object.id, name=perm)
                perms.delete()
        else:
            if not self.has_row_perm(instance, perm):
                return False
            content_type = ContentType.objects.get_for_model(instance)
            objects = Permission.objects.filter(group=self, content_type__pk=content_type.id, object_id=instance.id, name=perm)
            objects.delete()
        return True

    def has_row_perm(self, instance, perm):
        content_type = ContentType.objects.get_for_model(instance)
        objects = Permission.objects.filter(group=self, content_type__pk=content_type.id, object_id=instance.id, name=perm)
        if objects.count()>0:
            return True
        else:
            return False

    def get_rows_with_permission(self, instance, perm):
        content_type = ContentType.objects.get_for_model(instance)
        objects = Permission.objects.filter(group=self, content_type__pk=contet_type.id, name=perm)
        return objects




# Enhydris settings for {{ project_name }} project.
#
# Generated by 'enhydris-admin newinstance' using Enhydris {{ enhydris_version }}
# and Django {{ django_version }}.
#
# For more information on this file, see
# http://enhydris.readthedocs.org/en/{{ enhydris_docs_version }}/general/install.html#settings-reference

from enhydris.settings.base import *

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True
TEMPLATE_DEBUG = DEBUG

# SECURITY_WARNING: Keep the secret key used in production secret!
SECRET_KEY = '{{ secret_key }}'

# List the domains through which this instance will be accessible. Enhydris
# will refuse to serve other domains.
ALLOWED_HOSTS = ['example.com']

# Administrators who will be notified whenever attention is needed.
ADMINS = (
    # ('Your Name', 'your_email@domain.com'),
)
MANAGERS = ADMINS

# Whenever the system emails users, this is the address from which the emails
# will appear to have been sent.
DEFAULT_FROM_EMAIL = 'noreply@example.com'

# Whenever the system emails administrators, this is the address from which the
# emails will appear to have been sent.
SERVER_EMAIL = DEFAULT_FROM_EMAIL

DATABASES = {
    'default': {
        'ENGINE': 'django.contrib.gis.db.backends.postgis',
        'NAME': 'enhydris_db',
        'USER': 'enhydris_user',
        'PASSWORD': 'topsecret',
        'HOST': 'localhost',
        'PORT': '5432',
    }
}

# Default system timezone, such as 'Europe/Athens'.
TIME_ZONE = 'UTC'

# You should probably leave this as it is. See the installation instructions if
# in doubt.
SITE_ID = 1

# Where static files will be stored. By "static" files we mean css, javascript,
# and images of the skin. These files will be copied there at installation
# time when executing "python manage.py collectstatic".
STATIC_ROOT = '/var/cache/enhydris/static/'


# Where media files will be stored. "media" files are static files uploaded
# by users, e.g. images and videos of stations. The web server must be
# configured to map the URL /enhydris-media/ to that directory. See the
# installation instructions for more information.
MEDIA_ROOT = '/var/lib/enhydris/media/'

# The web server must be configured to map the URLs below to the directories
# specified by STATIC_ROOT and MEDIA_ROOT above. Enhydris will use the settings
# below to create appropriate links.
STATIC_URL = '/enhydris-static/'
MEDIA_URL = '/enhydris-media/'

# Mail server settings; used whenever the system needs to email users or
# admins.
EMAIL_USE_TLS = True
EMAIL_PORT = 587
EMAIL_HOST = 'smtp.example.com'
EMAIL_HOST_USER = 'emailuser'
EMAIL_HOST_PASSWORD = 'topsecret'

# The above are only the settings that are absolutely essential. Check the
# installation instructions for more settings that you can set.



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Database',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=64, blank=True)),
                ('ip_address', models.CharField(max_length=16, verbose_name='IP address')),
                ('hostname', models.CharField(unique=True, max_length=255, verbose_name='Hostname')),
                ('descr', models.TextField(max_length=255, verbose_name='Description', blank=True)),
                ('last_sync', models.DateTimeField(null=True, editable=False)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
    ]
